; Prepara uma lista em um espaço da memoria
; es:di = Lista
; cx = Tamanho em bytes que a lista deve ocupar
_ListaRemotoPreparar:
    push cx
    push di
    push ax
    
    push di
    push cx
    xor ax, ax
    rep stosb
    pop cx
    pop di
    
    shr cx, 1
    dec cx
    dec cx
    es mov [di], cx
    
    pop ax
    pop di
    pop cx
    retf

; Insere um item ao fim da lista
; es:di = Lista
; ax = Item
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
_ListaRemotoInserir:
    push di
    
    es mov bx, [di + 2]
    es cmp bx, [di]
    jb .continuar
        xor bx, bx
        clc
        jmp .fim
    .continuar:
    es inc word [di + 2]
    shl bx, 1
    es mov [di + bx + 4], ax
    shr bx, 1
    
    stc
    .fim:
    pop di
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      ax = Valor
_ListaRemotoLer:
    push di
    
    es cmp bx, [di + 2]
    jb .continuar
        xor ax, ax
        clc
        jmp .fim
    .continuar:
    shl bx, 1
    es mov ax, [di + bx + 4]
    shr bx, 1
    
    stc
    .fim:
    pop di
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      si = Valor
_ListaRemotoLerSI:
    push cs
    call _ListaRemotoLer
    push ax
    pop si
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      es:di = Valor:0
_ListaRemotoLerES:
    push cs
    call _ListaRemotoLer
    pushf
    push ax
    pop es
    xor di, di
    popf
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      ds:si = Valor:0
_ListaRemotoLerDS:
    push cs
    call _ListaRemotoLer
    pushf
    push ax
    pop ds
    xor si, si
    popf
    retf

; Remove um item da lista
; es:di = Lista
; bx = Posicao
; ret: cf = 1=Sucesso | 0 = Item nao existe
_ListaRemotoRemover:
    push di
    push si
    push ds
    push ax
    
    es mov bx, [di + 2]
    es cmp bx, [di]
    jb .continuar
        xor bx, bx
        clc
        jmp .fim
    .continuar:
    es dec word [di + 2]
    shl bx, 1
    es mov cx, [di + 2]
    sub cx, bx
    mov ax, es
    mov ds, ax
    mov si, di
    add si, 4
    add si, bx
    rep movsw
    
    stc
    .fim:
    pop ax
    pop ds
    pop si
    pop di
    retf

; Quantidade de itens em uma lista
; es:di = Lista
; ret: cx = Quantidade
_ListaRemotoQuantidade:
    es mov cx, [di + 2]
    retf

; Capacidade de itens de uma lista
; es:di = Lista
; ret: cx = Capacidade
_ListaRemotoCapacidade:
    es mov cx, [di]
    retf

; Tamanho em bytes de uma lista
; es:di = Lista
; ret: cx = Tamanho
_ListaRemotoBytes:
    es mov cx, [di]
    shl cx, 1
    add cx, 4
    retf

; Prepara uma lista em um espaço da memoria
; es:di = Lista
; cx = Tamanho em bytes que a lista deve ocupar
_ListaLocalPreparar:
    push cx
    push di
    push ax
    
    push di
    push cx
    xor ax, ax
    rep ds stosb
    pop cx
    pop di
    
    shr cx, 1
    dec cx
    dec cx
    ds mov [di], cx
    
    pop ax
    pop di
    pop cx
    retf

; Insere um item ao fim da lista
; es:di = Lista
; ax = Item
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
_ListaLocalInserir:
    push di
    
    ds mov bx, [di + 2]
    ds cmp bx, [di]
    jb .continuar
        xor bx, bx
        clc
        jmp .fim
    .continuar:
    ds inc word [di + 2]
    shl bx, 1
    ds mov [di + bx + 4], ax
    shr bx, 1
    
    stc
    .fim:
    pop di
    retf

; Ler um item ao fim da lista
; ds:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      ax = Valor
_ListaLocalLer:
    push di
    
    ds cmp bx, [di + 2]
    jb .continuar
        xor ax, ax
        clc
        jmp .fim
    .continuar:
    shl bx, 1
    ds mov ax, [di + bx + 4]
    shr bx, 1
    
    stc
    .fim:
    pop di
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      si = Valor
_ListaLocalLerSI:
    push cs
    call _ListaLocalLer
    push ax
    pop si
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      es:di = Valor:0
_ListaLocalLerES:
    push cs
    call _ListaLocalLer
    pushf
    push ax
    pop es
    xor di, di
    popf
    retf

; Ler um item ao fim da lista
; es:di = Lista
; ret: cf = 1=Sucesso | 0=Estouro da Lista
;      bx = Posicao
;      ds:si = Valor:0
_ListaLocalLerDS:
    push cs
    call _ListaLocalLer
    pushf
    push ax
    pop ds
    xor si, si
    popf
    retf

; Remove um item da lista
; es:di = Lista
; bx = Posicao
; ret: cf = 1=Sucesso | 0 = Item nao existe
_ListaLocalRemover:
    push di
    push si
    push ds
    push ax
    
    ds mov bx, [di + 2]
    ds cmp bx, [di]
    jb .continuar
        xor bx, bx
        clc
        jmp .fim
    .continuar:
    ds dec word [di + 2]
    shl bx, 1
    ds mov cx, [di + 2]
    sub cx, bx
    mov ax, es
    mov ds, ax
    mov si, di
    add si, 4
    add si, bx
    rep ds movsw
    
    stc
    .fim:
    pop ax
    pop ds
    pop si
    pop di
    retf

; Quantidade de itens em uma lista
; es:di = Lista
; ret: cx = Quantidade
_ListaLocalQuantidade:
    ds mov cx, [di + 2]
    retf

; Capacidade de itens de uma lista
; es:di = Lista
; ret: cx = Capacidade
_ListaLocalCapacidade:
    ds mov cx, [di]
    retf

; Tamanho em bytes de uma lista
; es:di = Lista
; ret: cx = Tamanho
_ListaLocalBytes:
    ds mov cx, [di]
    shl cx, 1
    add cx, 4
    retf
